@@ -44,6 +44,10 @@
 
 #define is_displayable(c) (((c) >= 0x20) && ((c) < 0x7f))
 
+#define GTKHEX_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object),\
+                                   GTK_TYPE_HEX,                         \
+                                   GtkHexPrivate))
+
 enum {
 	CURSOR_MOVED_SIGNAL,
 	DATA_CHANGED_SIGNAL,
@@ -72,6 +76,16 @@ struct _GtkHex_AutoHighlight
 	GtkHex_AutoHighlight *next, *prev;
 };
 
+struct _GtkHexPrivate
+{
+	/* Buffer for storing formatted data for rendering;
+	   dynamically adjusts its size to the display size */
+	guchar *disp_buffer;
+
+	gint default_cpl;
+	gint default_lines;
+};
+
 static gint gtkhex_signals[LAST_SIGNAL] = { 0 };
 
 static GtkFixedClass *parent_class = NULL;
@@ -691,7 +705,7 @@ render_hex_lines (GtkHex *gh,
 
 	gdk_cairo_set_source_rgba (cr, &fg_color);
 
-	frm_len = format_xblock(gh, gh->disp_buffer, (gh->top_line+imin)*gh->cpl,
+	frm_len = format_xblock (gh, gh->priv->disp_buffer, (gh->top_line+imin)*gh->cpl,
 							MIN((gh->top_line+imax+1)*gh->cpl, gh->document->file_size) );
 	
 	for(i = imin; i <= imax; i++) {
@@ -701,7 +715,7 @@ render_hex_lines (GtkHex *gh,
 
 		render_hex_highlights (gh, cr, i);
 		cairo_move_to (cr, 0, i * gh->char_height);
-		pango_layout_set_text (gh->xlayout, gh->disp_buffer + (i - imin)*xcpl, MIN(xcpl, tmp));
+		pango_layout_set_text (gh->xlayout, gh->priv->disp_buffer + (i - imin) * xcpl, MIN(xcpl, tmp));
 		pango_cairo_show_layout (cr, gh->xlayout);
 	}
 	
@@ -745,7 +759,7 @@ render_ascii_lines (GtkHex *gh,
 
 	gdk_cairo_set_source_rgba (cr, &fg_color);
 	
-	frm_len = format_ablock(gh, gh->disp_buffer, (gh->top_line+imin)*gh->cpl,
+	frm_len = format_ablock (gh, gh->priv->disp_buffer, (gh->top_line+imin)*gh->cpl,
 							MIN((gh->top_line+imax+1)*gh->cpl, gh->document->file_size) );
 	
 	for(i = imin; i <= imax; i++) {
@@ -756,7 +770,7 @@ render_ascii_lines (GtkHex *gh,
 		render_ascii_highlights (gh, cr, i);
 
 		cairo_move_to (cr, 0, i * gh->char_height);
-		pango_layout_set_text (gh->alayout, gh->disp_buffer + (i - imin)*gh->cpl, MIN(gh->cpl, tmp));
+		pango_layout_set_text (gh->alayout, gh->priv->disp_buffer + (i - imin)*gh->cpl, MIN(gh->cpl, tmp));
 		pango_cairo_show_layout (cr, gh->alayout);
 	}
 	
@@ -993,10 +1007,10 @@ static void recalc_displays(GtkHex *gh, guint width, guint height) {
 	xcpl = gh->cpl*2 + (gh->cpl - 1)/gh->group_type;
 	gh->xdisp_width = xcpl*gh->char_width;
 
-	if(gh->disp_buffer)
-		g_free(gh->disp_buffer);
+	if (gh->priv->disp_buffer)
+		g_free (gh->priv->disp_buffer);
 	
-	gh->disp_buffer = g_malloc( (xcpl + 1) * (gh->vis_lines + 1) );
+	gh->priv->disp_buffer = g_malloc ((xcpl + 1) * (gh->vis_lines + 1));
 
 	/* calculate new display position */
 	value = MIN (gh->top_line * old_cpl / gh->cpl, gh->lines - gh->vis_lines);
@@ -1710,8 +1724,8 @@ static void gtk_hex_real_paste_from_clipboard(GtkHex *gh)
 static void gtk_hex_finalize(GObject *o) {
 	GtkHex *gh = GTK_HEX(o);
 	
-	if(gh->disp_buffer)
-		g_free(gh->disp_buffer);
+	if (gh->priv->disp_buffer)
+		g_free (gh->priv->disp_buffer);
 
 	if (gh->disp_font_metrics)
 		pango_font_metrics_unref (gh->disp_font_metrics);
@@ -2011,11 +2025,11 @@ static void gtk_hex_size_request(GtkWidget *w, GtkRequisition *req) {
 
 	gtk_widget_get_preferred_size (gh->scrollbar, &sb_req, NULL);
 	req->width = 2 * padding.left + 2 * padding.right + 2 * gtk_container_get_border_width (GTK_CONTAINER (w)) +
-		sb_req.width + gh->char_width * (DEFAULT_CPL + (DEFAULT_CPL - 1) /
+		sb_req.width + gh->char_width * (gh->priv->default_cpl + (gh->priv->default_cpl - 1) /
 										 gh->group_type);
 	if(gh->show_offsets)
 		req->width += padding.left + padding.right + 8 * gh->char_width;
-	req->height = DEFAULT_LINES * gh->char_height + padding.top + padding.bottom +
+	req->height = gh->priv->default_lines * gh->char_height + padding.top + padding.bottom +
 		2*gtk_container_get_border_width(GTK_CONTAINER(w));
 }
 
@@ -2044,6 +2058,7 @@ gtk_hex_get_preferred_height (GtkWidget *widget,
 }
 
 static void gtk_hex_class_init(GtkHexClass *klass, gpointer data) {
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent(klass);
@@ -2102,19 +2117,24 @@ static void gtk_hex_class_init(GtkHexClass *klass, gpointer data) {
 	GTK_WIDGET_CLASS(klass)->key_release_event = gtk_hex_key_release;
 	GTK_WIDGET_CLASS(klass)->button_release_event = gtk_hex_button_release;
 
-	/* Changed in Gnome 2.0 -- SnM */
-	G_OBJECT_CLASS(klass)->finalize = gtk_hex_finalize;
+	object_class->finalize = gtk_hex_finalize;
 
 	parent_class = g_type_class_ref (gtk_fixed_get_type ());
+
+	g_type_class_add_private (object_class, sizeof (GtkHexPrivate));
 }
 
 static void gtk_hex_init(GtkHex *gh, gpointer klass) {
 	GtkCssProvider *provider;
 	GtkStyleContext *context;
 
+	gh->priv = GTKHEX_GET_PRIVATE (gh);
+	gh->priv->disp_buffer = NULL;
+	gh->priv->default_cpl = DEFAULT_CPL;
+	gh->priv->default_lines = DEFAULT_LINES;
+
 	gh->scroll_timeout = -1;
 
-	gh->disp_buffer = NULL;
 	gh->document = NULL;
 	gh->starting_offset = 0;
 
@@ -2583,32 +2603,8 @@ void gtk_hex_delete_autohighlight(GtkHex *gh, GtkHex_AutoHighlight *ahl)
 
 void gtk_hex_set_geometry(GtkHex *gh, gint cpl, gint vis_lines)
 {
-	gint width, height, xcpl;
-	GtkBorder padding;
-	GtkRequisition req;
-	GtkStateFlags state;
-	GtkStyleContext *context;
-
-	gtk_widget_get_preferred_size (gh->scrollbar, &req, NULL);
-
-	if(cpl <= 0 || vis_lines <= 0)
-		return;
-
-	context = gtk_widget_get_style_context (GTK_WIDGET (gh));
-	state = gtk_widget_get_state_flags (GTK_WIDGET (gh));
-	gtk_style_context_get_padding (context, state, &padding);
-
-	xcpl = 2*cpl + (cpl - 1)/gh->group_type;
-	width = xcpl*gh->char_width + cpl*gh->char_width;
-	width += 2 * gtk_container_get_border_width (GTK_CONTAINER (gh))
-	       + 2 * padding.left + 2 * padding.right +	req.width;
-	if(gh->show_offsets)
-		width += padding.left + padding.right + 8 * gh->char_width;
-
-	height = vis_lines*gh->char_height;
-	height += 2 * gtk_container_get_border_width (GTK_CONTAINER (gh)) + padding.top + padding.bottom;
-
-	gtk_widget_set_size_request(GTK_WIDGET(gh), width, height);
+    gh->priv->default_cpl = cpl;
+    gh->priv->default_lines = vis_lines;
 }
 
 void add_atk_namedesc (GtkWidget *widget, const gchar *name, const gchar *desc)
